#!/usr/bin/perl -w
# $Id: prepconflist.pl,v 1.7 2005/09/25 21:43:10 oliver Exp $

use strict;
#use FindBin;
#use lib $FindBin::Bin; # Messages.pm is in script directory
                       # add other lib dirs in environment var PERL5LIB
use lib "/sansom/gfio/oliver/Devel/Setup/UmbrellaSampling";
use Messages; 

use Getopt::Long;

my $Version_ID = q($Id: prepconflist.pl,v 1.7 2005/09/25 21:43:10 oliver Exp $ );

# imported from Messages
# DEBUG level says how chatty we are:
#  <0  dead silent
#   0  no diagnostic messages, only output
#   1  a few additional important informations
#   3  verbose
#   5  extra verbose
#  10  start with real debugging stuff
#  15  many intermediate results
#  16  print input files line-by-line
#  20  everything   
$DEBUG = 1;

my $STARTDIR="./start";
my $OutputFn = "conf.list";
my $PULLATOM = "NA";
my $DataFn;
my $APPEND_LIST;
my ($Z1,$Z2,$RADIUS,$XCOM,$YCOM,$ZCOM);
my %cylinder = ("R" => 0, "cpoint" => [0,0,0], "axis" => [0,0,1], "ends" => [0,0]);

my %requiredfiles = ();
my %requiredvars = ( "--radius"      => \$RADIUS,
                     "--xcom"        => \$XCOM,
                     "--ycom"        => \$YCOM,
                     "--z1"          => \$Z1,
                     "--z2"          => \$Z2
                     );

my ($opt_verbose, $opt_version, $opt_help);

# how to indicate a comment line in the data file (regex)
my $commentchar = '[;#@]';



######################################
#
# MAIN
#
######################################

&parseArgs;
if ($opt_help) {&print_usage; exit 0};
if ($opt_version) { msg (0, "%s\n", $Version_ID); exit 0};
&checkArgs;
&prepArgs;


# reading a gro file
# 1 comment
# 2 number of atoms
# data lines
my $template = "A5 A5 A5 A5 A8 A8 A8 A8 A8 A8 A1";
my ($resnr,$resname,$atomname,$atomnr,$x,$y,$z,$vx,$vy,$vz);
my $buf = "";
my %n_hits = ("total" => 0, "file" => 0);
my @conflist;   # holds the full list, sorted at the end and written to file

foreach $DataFn (@ARGV) {
    open GRO, "< $DataFn" or die "File $DataFn not found,";
    logger(3,"Processing file '$DataFn':");
    $n_hits{"file"} = 0;
    while (<GRO>) {
        chomp;
        # only process atom entries
        #    1234ATOM ....
        next if (! /^ *[0-9]+[a-zA-Z]+/);

        # get data from fixed format gro file
        # and remove all spaces
        { 
            my @entry = unpack($template,$_);
            foreach (@entry) { tr/ //d; };
            ($resnr,$resname,$atomname,$atomnr,$x,$y,$z,$vx,$vy,$vz) = @entry;
        };
        # only interested in 
        # (1) residues with name PULLATOM
        # (2) which have the prescribed z coordinate
        next unless (($atomname =~ $PULLATOM) && (&bInCylinder($x,$y,$z,\%cylinder)));
#        printf LIST "%-20s %-3s %6d %.3f %.3f %.3f\n", 
#               $DataFn, $atomname,$atomnr,$x,$y,$z;
        push @conflist,  [$DataFn, $atomname,$atomnr,$x,$y,$z];
        ++$n_hits{"file"}; ++$n_hits{"total"};

        if ($DEBUG > 6) {
            if ( $vx && $vy && $vz ) {
                $buf = sprintf("%5d%-5s%5s%5d%8.3f%8.3f%8.3f%8.4f%8.4f%8.4f\n",
                               $resnr,$resname,$atomname,$atomnr,
                               $x,$y,$z,$vx,$vy,$vz);
            } else {
                $buf = sprintf("%5d%-5s%5s%5d%8.3f%8.3f%8.3f\n",
                               $resnr,$resname,$atomname,$atomnr,
                               $x,$y,$z);
            };
            logger(6,"[%s] %s",$DataFn,$buf);
        };
    }
    close(GRO);
    logger(3," %d hits\n",$n_hits{"file"});
}



if ($APPEND_LIST) {
    open LIST, ">> $OutputFn" or die "Cannot open $OutputFn to append,";
} else {
    open LIST, "> $OutputFn" or die "Cannot open $OutputFn,";
}

# sort by z (which is the 6th position)
foreach (sort { $a->[5] <=> $b->[5] } @conflist) {
    printf LIST "%-20s %-3s %6d %.3f %.3f %.3f\n", @$_;
}
close(LIST);
if ($APPEND_LIST) {
logger(3,"Appended %d matching positions for $PULLATOM to $OutputFn.\n",
       $n_hits{"total"});
} else {
logger(3,"Created $OutputFn with %d matching positions for $PULLATOM.\n",
       $n_hits{"total"});
}


exit(0);	
# end main ()

sub bInCylinder ($$$$) {
    my ($x,$y,$z,$c) = @_;
    return (($z >= $c->{"ends"}[0] && $z <= $c->{"ends"}[1])
            && (($x-$c->{"cpoint"}[0])**2 + ($y-$c->{"cpoint"}[1])**2 <= $c->{"R"}**2))
}

sub print_usage () {
    my $Prog = &Progname();
    print <<EOF;
usage: $Prog [OPTIONS] file1.gro [file2.gro ...]

Create the file list which forms the major content of the input file
('conf.dat') to prepumbrella.pl. It reads all files given on the command
line and writes matches to the output file (with --append it appends to the
outout file). 

Typical usage: 

 find start/ -name \\*.gro | xargs $Prog [OPTIONS]

See http://indigo1.biop.ox.ac.uk/wiki/index.php/PMF_test_runs#Input_file_conf.dat_for_prepumbrella.pl

The OPTIONS can be abbreviated; defaults are given in [].	 
--help              help
--verbose           (alias for DEBUGLEVEL=3)
--version           print version id
--debug=DEBUGLEVEL  0 (almost) quiet, <0 silent,
                    1..3 verbose, 
                    >5 really debugging stuff (20 max) [$DEBUG]

--output=FILE       output gro file [$OutputFn]
--append            append to output file (do not overwrite)

--atomname=STRING   name of the pulled atom in the gro file [$PULLATOM]
--z1=FLOAT          only list starting positions with z coordinate   
--z2=FLOAT          between z1 and z2 (in nm).                       
--radius=FLOAT      Also, only list them when they are within a cylinder
--xcom=FLOAT        of radius R which is centered on the center of mass
--ycom=FLOAT        (give x and y coordinates), pointing along z.
--zcom=FLOAT        (not necessary at the moment: the cylinder is always
                    parallel to the z-axis)
EOF
}


sub parseArgs () {
    &GetOptions( "output=s" => \$OutputFn,
                 "append"   => \$APPEND_LIST,
                 "atomname=s"=> \$PULLATOM,
                 "z1=f"     => \$Z1,
                 "z2=f"     => \$Z2,
                 "radius=f" => \$RADIUS,
                 "xcom=f"   => \$XCOM,
                 "ycom=f"   => \$YCOM,
                 "zcom=f"   => \$ZCOM,
                 "debug=i" => \$DEBUG,
                 "verbose" => \$opt_verbose,
                 "version" => \$opt_version,
                 "help" => \$opt_help) or &print_usage;
    if ($opt_verbose) { $DEBUG = 3 };
}

sub checkArgs () {
    my $argerror = 0;
    my $opt;

    foreach $opt (keys %requiredfiles) {
        my $file = ${$requiredfiles{$opt}};
        next if (-e $file);
        $argerror = 2;
        errmsg("Required file `%s', set with option %s,  not found.\n",
               $file,$opt);
    };

    foreach $opt (keys %requiredvars) {
        my $var = ${$requiredvars{$opt}};
        next if ($var);
        $argerror = 3;
        errmsg("Required parameter associated with option %s was not set.\n",
               $opt);
    }

    if ($argerror && $DEBUG < 10) {
        errmsg(" EXIT. Type ".&Progname." --help for the correct syntax.\n");
        exit($argerror);
    };    
}

sub prepArgs {
    $cylinder{"R"} = $RADIUS;
    $cylinder{"cpoint"} = [$XCOM,$YCOM,$ZCOM];
    $cylinder{"axis"} = [0,0,1];           # parallel z fixed
    $cylinder{"ends"} = [$Z1,$Z2];         # length along axis in general
}
